function [mpP, function_ID]=GetDiameter_Episodic_L5_cc(fname,matfilename,trialboundaries,window_time_size,the_x,the_decimate, the_objective,user_microns_per_pixel,channel,the_scanmirrors);
%funtion for loading *.MPD linescan files and performing the Radon transform on
%them in order to get the RBC velocity
%it breaks up files into stimulus triggered episodes
%
%becase MPScan gets line scans in both directions, the frame is split into even
%and odd lines and velocity is extravted independently from each one and then averaged
%INPUTS:
%fname-filename
%trialboundaries- vector with start frames of episodes
%in a trials

%windowtimesize - size (in sec) of the advancemt of the Radon window
%anglespan - how many degrees (+/-) to look in the adaptive search
%angle_resolution - fine angle resolution
%the_x - {[left  right]} cell array of bounds of which to extract from the frames in the X
%dimension
%the_decimate - factor by which to decimate the line scan in the X dimension
%max_v - time points above this absolute speed are clipped out and interpolated
%between the last and next valid time points

%OUTPUTS
%fixed_v - velocities of forward and back linescans
%the_t - times
%mpP -  a structure with the relevant data
%mpP.Header substruct
%mpP.Ch1 first two frames from channel 1
%mpP.Diameter structure with diameter info etc.

%Open stream
function_ID = mfilename('fullpath');
figure(1)
close(1)
f1 = figure(1);
set(f1,'visible','on');
mpfile = actxcontrol('MPfile.Data',[0,0,500,500],f1);
OpenFile = invoke(mpfile,'OpenMPFile',fname);

%Read header and take further action based on header information
mpP.Header = mp2mat_readHeader(mpfile,fname);
mpP.num_frames = max(trialboundaries)-min(trialboundaries)+1;%theframes(end)-theframes(1);%str2double(mpContent.Header.Frame_Count);
mpP.xsize = str2double(mpP.Header.Frame_Width);
mpP.ysize = str2double(mpP.Header.Frame_Height);
frame_height=mpP.ysize;
for chNum = 1%1 : 4
    chNumStr = num2str(chNum);
    if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
        mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[1 2],the_x);
    end
end
if length(matfilename)>0
    load(matfilename)
    mpP.scanData=scanData;
else
    mpP.scan_path=[];
end
channel=1;
first_frames=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, 2:11,the_x);%get first 10 frames for background subtraction

time_per_line=str2num(mpP.Header.Frame_Width)*(5/4)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
if strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    time_per_line=str2num(mpP.Header.Frame_Width)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
end
mpP.Header.LineRate=1/time_per_line;
Fs_Blood_Flow=mpP.Header.LineRate;
windowsizea=round(Fs_Blood_Flow*window_time_size);%*.25 %for interlevaed lines
%adapt windowsize to
windowsize=4*round(windowsizea/4);
mpP.Diameter.windowsize=windowsize;

%%%%basic stuff
nframes=mpP.Header.Frame_Count;%theframes(2)-theframes(1)+1;
stepsize=.25*windowsize;
nlines=nframes*str2num(mpP.Header.Frame_Height);
npoints=max(the_x)-min(the_x)+1%str2num(mpP.Header.Frame_Width);

ntrials=length(trialboundaries)
n_frames_trial=trialboundaries(2)-trialboundaries(1);
n_lines_trial= n_frames_trial*mpP.ysize;

%%
for trial=1:ntrials
    nsteps=floor(n_lines_trial/stepsize)-3;

    data_var=zeros(nsteps,1);
    data_hold=zeros(windowsize,npoints);

    %data caching

    %n_lines_trial
    nframes_to_cache=n_frames_trial;
    frame_counter=1+(trial-1)*n_frames_trial;
    line_counter=3*stepsize;
    lines_per_frame=mpP.ysize;
    cached_lines=1:(nframes_to_cache*mpP.ysize);
    use_lines=1:windowsize;
    data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,1,mpP, [(frame_counter) (frame_counter+nframes_to_cache-1)],the_x);
    data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
    clear data_cache_folded
    mpP.Diameter.Image{trial}=data_cache;

    data_hold=zeros(1,windowsize,length([the_x(1):the_decimate:the_x(2)-1]));
    data_hold(1,:,:)=double(data_cache(1:windowsize,1:the_decimate:end-1));
    %data_hold(2,:,:)=double(data_cache(2:2:windowsize,end:-the_decimate:2));


    for chNum = 3%1 : 4
        chNumStr = num2str(chNum);
        if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
            mpP.ECoG.trials{trial} = mp2mat_getChannelData_Episodic(mpfile,chNum,mpP,[(frame_counter) (frame_counter+nframes_to_cache-1)],the_x,diff(trialboundaries(1:2)));
        end
    end


    frame_counter=frame_counter+nframes_to_cache;

    %%

    for k=1:nsteps
        line_counter=line_counter+stepsize;
        frame_line_marker=find(cached_lines==line_counter+1);%find the new place in the frame
        %%%check to see if the data hold will overrun the cache
        n_overrunlines=line_counter-max(cached_lines)+stepsize;%do we run over?
        if(n_overrunlines<0)%keep going using the currently loaded frames
            data_hold(1,:,:)=[squeeze(data_hold(1,stepsize+1:end,:)); (double(data_cache(frame_line_marker:frame_line_marker+stepsize-1,1:the_decimate:end-1)))];
            %   data_hold(2,:,:)=[squeeze(data_hold(2,stepsize/2+1:end,:)); (double(data_cache(frame_line_marker+1:2:frame_line_marker+stepsize,end:-the_decimate:2)))];
        else
            disp(['noverunline=' num2str(n_overrunlines)])%overrun!!!!!!!!!!1!!1!!')
        end
        use_lines=use_lines+stepsize;
        %take out the local mean of the window
        the_t(k)=1+(k-1)*stepsize+windowsize/2;
        npoints_decimated=size(data_hold,3);
        data_hold_mean=mean(double(squeeze(data_hold(1,:,:))));%average all the data in the window in order to estimate diameter
        mpP.Diameter.raw_diameter(trial,k)=calcFWHM(data_hold_mean);
        mpP.Diameter.image_sum{trial}(k,:)=data_hold_mean;
    end

    %%
    %thetas=thetas-90;

    %objective_mag=1/461.3;%measured on 40x olympus dec12 2007 pd
%     if (the_objective==1)&&(the_scanmirrors==1)%40x Olympus
%         microns_per_pixel=1/1.5;%(str2num(mpP.Header.Frame_Width));
%     elseif (the_objective==1)&&(the_scanmirrors==2)
%         microns_per_pixel=.899/1.5;
%     elseif (the_objective==2)&&(the_scanmirrors==1);
%         microns_per_pixel=1.59;%(str2num(mpP.Header.Frame_Width));
%     elseif (the_objective==2)&&(the_scanmirrors==2);
%         microns_per_pixel=1.43;%(str2num(mpP.Header.Frame_Width));
%     end
    %microns_per_pixel=1/1.5;%(str2num(mpP.Header.Frame_Width));
    microns_per_pixel=user_microns_per_pixel;
    if strcmp(mpP.Header.Scan_Mode,'Line Scan')
        Xfactor=microns_per_pixel/(str2num(mpP.Header.Magnification(2:end)));%objective_mag*microns_per_pixel/str2num(mpP.Header.Magnification(2:end));
        Tfactor =mpP.Header.LineRate;
    elseif  strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
        Xfactor=microns_per_pixel*((mpP.scanData.scanVelocity)/(4/512));%4volts/512
        Tfactor=mpP.Header.LineRate;
    end
    mpP.Xfactor=Xfactor;
    mpP.Tfactor=Tfactor;
    mpP.microns_per_pixel=microns_per_pixel;
    mpP.Diameter.the_decimate=the_decimate;
    mpP.Diameter.diameter(trial,:)=the_decimate*Xfactor*mpP.Diameter.raw_diameter(trial,:);
    the_date=date;
    mpP.Diameter.stimulus(1:ntrials)=0;
    diameter=mpP.Diameter.diameter;
    try %to load framedata
        data_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache-1)],the_x);
        mpP.Image{trial}=LinescanFramesUnfoldMPScan(data_folded)';
    catch
        mpP.Image{trial}=[];%zeros(mpP.num_frames*str2num(mpP.Header.Frame_Height),1);
    end
    try
        for chNum = 3%1 : 4
            chNumStr = num2str(chNum);
            if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
                mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[min(trialboundaries) max(trialboundaries)],the_x);
            end
        end
        mpP.ECoG.Fs=1/length(mpP.Ch3)/str2num(mpP.Header.Frame_Count)*((1/mpP.Header.LineRate)/str2num(mpP.Header.Frame_Height));
    end

end
save(['mpP_' fname(1:end-4) '_' the_date],'mpP')
