function [mpP,function_ID]=GetVelocityRadon_Episodic_L4_cc(fname,matfilename,trialboundaries,window_time_size,angle_span, angle_resolution,the_x,the_decimate, max_v,user_microns_per_pixel,channel);
%funtion for loading *.MPD linescan files and performing the Radon transform on
%them in order to get the RBC velocity
%it breaks up files into stimulus triggered episodes
%
%becase MPScan gets line scans in both directions, the frame is split into even
%and odd lines and velocity is extravted independently from each one and then averaged 
%INPUTS:
%fname-filename
%trialboundaries- vector with start frames of episodes in a trial

%windowtimesize - size (in sec) of the advancemt of the Radon window 
%anglespan - how many degrees (+/-) to look in the adaptive search
%angle_resolution - fine angle resolution 
%the_x - {[left  right]} cell array of bounds of which to extract from the frames in the X
%dimension
%the_decimate - factor by which to decimate the line scan in the X dimension
%max_v - time points above this absolute speed are clipped out and interpolated
%between the last and next valid time points 


%OUTPUTS
%mpP -  a structure with the relevant data
%mpP.Header substruct
%mpP.Ch1 first two frames from channel 1
%mpP.Blood_flow structure with velocity info etc.
%function_ID - this function's location

%Open stream
function_ID = mfilename('fullpath');
figure(1)
close(1)
f1 = figure(1);
set(f1,'visible','on');
mpfile = actxcontrol('MPfile.Data',[0,0,500,500],f1);
OpenFile = invoke(mpfile,'OpenMPFile',fname);

%Read header and take further action based on header information
mpP.Header = mp2mat_readHeader(mpfile,fname);
mpP.num_frames = max(trialboundaries)-min(trialboundaries)+1;%theframes(end)-theframes(1);%str2double(mpContent.Header.Frame_Count);
mpP.xsize = str2double(mpP.Header.Frame_Width);
mpP.ysize = str2double(mpP.Header.Frame_Height);
frame_height=mpP.ysize;
for chNum = 1%1 : 4
   chNumStr = num2str(chNum);
   if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
       mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[1 2],the_x);
   end
end
if length(matfilename)>0
    load(matfilename);
    mpP.scanData=scanData;
else
    mpP.scan_path=[];
end

first_frames=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, 2:11,the_x);%get first 10 frames for background subtraction


if strcmp(mpP.Header.Scan_Mode,'Line Scan')
time_per_line=str2num(mpP.Header.Frame_Width)*(5/4)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
isinterleaved=1;
end
if strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    time_per_line=str2num(mpP.Header.Frame_Width)*(str2num(mpP.Header.Pixel_Clock)*(.05*1e-6));
    isinterleaved=0; %arb scans are not interleace 
end

mpP.Header.LineRate=1/time_per_line;
Fs_Blood_Flow=mpP.Header.LineRate;
windowsizea=round(Fs_Blood_Flow*window_time_size);%*.25 %for interlevaed lines
%adapt windowsize to 
if isinterleaved==1
windowsize=8*round(windowsizea/8);
else
windowsize=4*round(windowsizea/4);   
end
 mpP.Blood_flow.windowsize=windowsize;

%%%%basic stuff 
nframes=mpP.Header.Frame_Count;%theframes(2)-theframes(1)+1;
stepsize=.25*windowsize;
nlines=nframes*str2num(mpP.Header.Frame_Height);
npoints=max(the_x)-min(the_x)+1;%str2num(mpP.Header.Frame_Width);

ntrials=length(trialboundaries);
n_frames_trial=trialboundaries(2)-trialboundaries(1);
n_lines_trial= n_frames_trial*mpP.ysize;

%%
for trial=1:ntrials
nsteps=floor(n_lines_trial/stepsize)-3;
angles=(1:180);%parameters for initial search
angles_baseline=0:15:179;
angles_adaptive=[-angle_span:1:angle_span];
angles_fine=-1.75:angle_resolution:1.75;%fine grained search
n_adaptive_angles=length(angles_adaptive);
theta_low=5;%boundaries for for adaptivce theta
theta_high=n_adaptive_angles-5;
spread_matrix=zeros(1,length(angles));
spread_matrix_adaptive=zeros(2,nsteps,length(angles_adaptive));
baseline_var=zeros(2,nsteps,length(angles_baseline));
spread_matrix_fine=zeros(2,nsteps,length(angles_fine));
thetas=zeros(2,nsteps);%the angle of the lines
data_var=zeros(nsteps,1);
data_hold=zeros(windowsize,npoints);

%data caching

%n_lines_trial    
nframes_to_cache=n_frames_trial;
frame_counter=1+(trial-1)*n_frames_trial;
line_counter=3*stepsize;
lines_per_frame=mpP.ysize;
cached_lines=1:(nframes_to_cache*mpP.ysize);
use_lines=1:windowsize;
data_cache_folded=mp2mat_getChannelData_narrow_noplot(mpfile,1,mpP, [(frame_counter) (frame_counter+nframes_to_cache-1)],the_x);
data_cache=(LinescanFramesUnfold(data_cache_folded,nframes_to_cache));
mpP.Blood_flow.Image{trial}=data_cache;
if isinterleaved==1
data_hold=zeros(2,windowsize/2,length([the_x(1):the_decimate:the_x(2)-1]));
data_hold(1,:,:)=double(data_cache(1:2:windowsize,1:the_decimate:end-1));
data_hold(2,:,:)=double(data_cache(2:2:windowsize,end:-the_decimate:2));
else
data_hold=zeros(1,windowsize,length([the_x(1):the_decimate:the_x(2)-1]));
data_hold(1,:,:)=double(data_cache(1:windowsize,1:the_decimate:end-1));

end

for chNum = 3%1 : 4
   chNumStr = num2str(chNum);
   if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
       mpP.ECoG.trials{trial} = mp2mat_getChannelData_Episodic(mpfile,chNum,mpP,[(frame_counter) (frame_counter+nframes_to_cache-1)],the_x,diff(trialboundaries(1:2)));
   end
end


frame_counter=frame_counter+nframes_to_cache;

data_cachemean_forward=mean(data_cache); 
if isinterleaved==1
data_cachemean_backward=(fliplr(data_cachemean_forward)); 
end

%%
if isinterleaved==1
for k=1:nsteps
    line_counter=line_counter+stepsize;
    frame_line_marker=find(cached_lines==line_counter+1);%find the new place in the frame
%%%check to see if the data hold will overrun the cache
    n_overrunlines=line_counter-max(cached_lines)+stepsize;%do we run over?

    if(n_overrunlines<0)%keep going using the currently loaded frames 
        data_hold(1,:,:)=[squeeze(data_hold(1,stepsize/2+1:end,:)); (double(data_cache(frame_line_marker:2:frame_line_marker+stepsize-1,1:the_decimate:end-1)))];
        data_hold(2,:,:)=[squeeze(data_hold(2,stepsize/2+1:end,:)); (double(data_cache(frame_line_marker+1:2:frame_line_marker+stepsize,end:-the_decimate:2)))];
    else
        disp(['noverunline=' num2str(n_overrunlines)])%overrun!!!!!!!!!!1!!1!!')
    end
    use_lines=use_lines+stepsize;
    %take out the local mean of the window
    the_t(k)=1+(k-1)*stepsize+windowsize/2;
    npoints_decimated=size(data_hold,3);
    for n=1:npoints_decimated
    data_hold_ms(1,:,n)=data_hold(1,:,n)-data_cachemean_forward(n);
    data_hold_ms(2,:,n)=data_hold(2,:,n)-data_cachemean_backward(n); 
    end
    data_hold_ms=data_hold_ms-mean(data_hold_ms(:));
    if k==1
    radon_hold(1,:,:)=(radon(squeeze(data_hold_ms(1,:,:)),angles));
    radon_hold(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),angles);
    spread_matrix(1,:)=var(squeeze(radon_hold(1,:,:)));
    spread_matrix(2,:)=var(squeeze(radon_hold(2,:,:)));
    [m(1,:) the_theta(1,:)]=max(spread_matrix(1,k));
    [m(2,:) the_theta(2,:)]=max(spread_matrix(2,k));
    thetas(1,k)=angles(the_theta(1,:));
    thetas(2,k)=angles(the_theta(2,:));
    radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),-thetas(1,k)+angles_adaptive);
    radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),-thetas(2,k)+angles_adaptive);
    spread_matrix_adaptive(1,k,1)=max(var(squeeze(radon_hold_adaptive(1,:,:))));
    spread_matrix_adaptive(2,k,1)=max(var(squeeze(radon_hold_adaptive(2,:,:))));
    radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)-mean(data_hold_ms(:))),thetas(1,k)+angles_fine);
    radon_hold_fine(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)-mean(data_hold_ms(:))),thetas(2,k)+angles_fine);
    spread_matrix_fine(1,k,:)=squeeze(var(squeeze(radon_hold_fine(1,:,:))));
    spread_matrix_fine(2,k,:)=squeeze(var(squeeze(radon_hold_fine(2,:,:))));
    [m the_theta(1,:)]=max(squeeze(spread_matrix_fine(1,k,:)));
    [m the_theta(2,:)]=max(squeeze(spread_matrix_fine(2,k,:)));
    thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
    thetas(2,k)=thetas(2,k)+angles_fine(the_theta(2,:));
    baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));
    baseline_var(2,k,:)=var(radon(squeeze(data_hold_ms(2,:,:)),angles_baseline));
    else        
    thetas(1,k)=thetas(1,k-1);
    thetas(2,k)=thetas(2,k-1);
    radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k-1)+angles_adaptive);
    radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),thetas(2,k-1)+angles_adaptive);
    spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));
    spread_matrix_adaptive(2,k,:)=var(squeeze(radon_hold_adaptive(2,:,:)));
    [m the_theta(1,:)]=max(spread_matrix_adaptive(1,k,:));
    [m the_theta(2,:)]=max(spread_matrix_adaptive(2,k,:));
    baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%figure out the average radon of the section for calculating S/N 
    baseline_var(2,k,:)=var(radon(squeeze(data_hold_ms(2,:,:)),angles_baseline));%figure out the average radon of the section for calculating S/N 
    %if the peak of the variance is at an extreme, redo for angles centered
    %around the peak

    while ((the_theta(1,:)<=theta_low)||(the_theta(1,:)>=theta_high))
        thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
        radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)), thetas(1,k)+angles_adaptive);
        spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));        
        [m the_theta(1,:)]=max(squeeze(spread_matrix_adaptive(1,k,:)));
    end
    while ((the_theta(2,:)<=theta_low)||(the_theta(2,:)>=theta_high))
        thetas(2,k)=thetas(2,k)+angles_adaptive(the_theta(2,:));
        radon_hold_adaptive(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)), thetas(2,k)+angles_adaptive);
        spread_matrix_adaptive(2,k,:)=var(squeeze(radon_hold_adaptive(2,:,:)));        
        [m the_theta(2,:)]=max(squeeze(spread_matrix_adaptive(2,k,:)));
    end
       
    baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%
    baseline_var(2,k,:)=var(radon(squeeze(data_hold_ms(2,:,:)),angles_baseline));%
    thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
    radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k)+angles_fine);
    spread_matrix_fine(1,k,:)=var(squeeze(radon_hold_fine(1,:,:)));
    [m the_theta(1,:)]=max(spread_matrix_fine(1,k,:));
    thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
    thetas(2,k)=thetas(2,k)+angles_adaptive(the_theta(2,:));
    radon_hold_fine(2,:,:)=radon(squeeze(data_hold_ms(2,:,:)),thetas(2,k)+angles_fine);
    spread_matrix_fine(2,k,:)=var(squeeze(radon_hold_fine(2,:,:)));
    [m the_theta(2,:)]=max(spread_matrix_fine(2,k,:));
    thetas(2,k)=thetas(2,k)+angles_fine(the_theta(2,:));
    end
end
end
%%
if isinterleaved==0 %non interleved data
for k=1:nsteps
    line_counter=line_counter+stepsize;
    frame_line_marker=find(cached_lines==line_counter+1);%find the new place in the frame
%%%check to see if the data hold will overrun the cache
    n_overrunlines=line_counter-max(cached_lines)+stepsize;%do we run over?

    if(n_overrunlines<0)%keep going using the currently loaded frames 
        data_hold(1,:,:)=[squeeze(data_hold(1,stepsize+1:end,:)); (double(data_cache(frame_line_marker:frame_line_marker+stepsize-1,1:the_decimate:end-1)))];
    else
        disp(['noverunline=' num2str(n_overrunlines)])%overrun!!!!!!!!!!1!!1!!')
    end
    use_lines=use_lines+stepsize;
    the_t(k)=1+(k-1)*stepsize+windowsize/2;
    npoints_decimated=size(data_hold,3);
    for n=1:npoints_decimated
    data_hold_ms(1,:,n)=data_hold(1,:,n)-data_cachemean_forward(n);
    end
    data_hold_ms=data_hold_ms-mean(data_hold_ms(:));
    if k==1
    radon_hold(1,:,:)=(radon(squeeze(data_hold_ms(1,:,:)),angles));
    spread_matrix(1,:)=var(squeeze(radon_hold(1,:,:)));
    [m(1,:) the_theta(1,:)]=max(spread_matrix(1,k));
    thetas(1,k)=angles(the_theta(1,:));
    radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),-thetas(1,k)+angles_adaptive);
    spread_matrix_adaptive(1,k,1)=max(var(squeeze(radon_hold_adaptive(1,:,:))));
    radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)-mean(data_hold_ms(:))),thetas(1,k)+angles_fine);
    spread_matrix_fine(1,k,:)=squeeze(var(squeeze(radon_hold_fine(1,:,:))));

    [m the_theta(1,:)]=max(squeeze(spread_matrix_fine(1,k,:)));
    thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));
    baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));
    else        
    thetas(1,k)=thetas(1,k-1);
    radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k-1)+angles_adaptive);
    spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));
    [m the_theta(1,:)]=max(spread_matrix_adaptive(1,k,:));
    baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%figure out the average radon of the section for calculating S/N 

    while ((the_theta(1,:)<=theta_low)||(the_theta(1,:)>=theta_high))
        thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
        radon_hold_adaptive(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)), thetas(1,k)+angles_adaptive);
        spread_matrix_adaptive(1,k,:)=var(squeeze(radon_hold_adaptive(1,:,:)));        
        [m the_theta(1,:)]=max(squeeze(spread_matrix_adaptive(1,k,:)));
    end
    baseline_var(1,k,:)=var(radon(squeeze(data_hold_ms(1,:,:)),angles_baseline));%
    thetas(1,k)=thetas(1,k)+angles_adaptive(the_theta(1,:));
    radon_hold_fine(1,:,:)=radon(squeeze(data_hold_ms(1,:,:)),thetas(1,k)+angles_fine);
    spread_matrix_fine(1,k,:)=var(squeeze(radon_hold_fine(1,:,:)));
    [m the_theta(1,:)]=max(spread_matrix_fine(1,k,:));
    thetas(1,k)=thetas(1,k)+angles_fine(the_theta(1,:));

    end
end
end
%%
%thetas=thetas-90;
if((length(find(thetas<0))>=1)&&(length(find(thetas>0))>=1))
    disp('sign flip!')
end
    

mpP.Blood_flow.thetas=thetas;

mpP.Blood_flow.unscaled_velocity{trial}=(cotd(thetas-90));
mpP.Blood_flow.spread_matrix_adaptive{trial}=spread_matrix_adaptive;
mpP.Blood_flow.baseline_var{trial}=baseline_var;

microns_per_pixel=user_microns_per_pixel;
if strcmp(mpP.Header.Scan_Mode,'Line Scan')
    Xfactor=microns_per_pixel/(str2num(mpP.Header.Magnification(2:end)));%objective_mag*microns_per_pixel/str2num(mpP.Header.Magnification(2:end));
    Tfactor =mpP.Header.LineRate;
elseif  strcmp(mpP.Header.Scan_Mode,'Arbitrary Scan')
    Xfactor=microns_per_pixel*((mpP.scanData.scanVelocity)/(4/512));%4volts/512 pixels
    Tfactor=mpP.Header.LineRate;
end
mpP.Xfactor=Xfactor;
mpP.Tfactor=Tfactor;
mpP.microns_per_pixel=microns_per_pixel;
mpP.Blood_flow.the_decimate=the_decimate;
if isinterleaved==1
mpP.Blood_flow.v{trial}(1,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(1,:)-90))/2;
mpP.Blood_flow.v{trial}(2,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(2,:)-90))/2;
else    %angles are the same way
mpP.Blood_flow.v{trial}(1,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(1,:)-90));
mpP.Blood_flow.v{trial}(2,:)=the_decimate*Tfactor*Xfactor*(cotd(thetas(1,:)-90));
end
mpP.Blood_flow.velocity2{trial}(1,:)=VelocityCleanUp(mpP.Blood_flow.v{trial}(1,:),max_v);
if isinterleaved==1
mpP.Blood_flow.velocity2{trial}(2,:)=VelocityCleanUp(mpP.Blood_flow.v{trial}(2,:),max_v);
else
mpP.Blood_flow.velocity2{trial}(2,:)=-(mpP.Blood_flow.velocity2{trial}(1,:));
end


v_hold=mpP.Blood_flow.velocity2{trial};
v_out=0.5*(v_hold(1,:)-v_hold(2,:));
mpP.Blood_flow.v_out(trial,:)=v_out;
mpP.Blood_flow.the_t=the_t/(mpP.Header.LineRate);
mpP.Blood_flow.the_Fs=4/window_time_size;

[fixed_v]=VelocityCleanUpSTDOutliers(mpP.Blood_flow.v_out(trial,:), 3);%
mpP.Blood_flow.fixed_v(trial,:)=fixed_v/1000;
the_date=date;
mpP.Bloodflow.stimulus(1:ntrials)=0;

seperability=max(squeeze(mean(mpP.Blood_flow.spread_matrix_adaptive{trial},1))')./mean(squeeze(mean(mpP.Blood_flow.baseline_var{trial},1))');%
mpP.Blood_flow.seperability(trial,:)=seperability;
sep_threshold=3;
low_sep=find(seperability<sep_threshold);

try %to load framedata
data_folded=mp2mat_getChannelData_narrow_noplot(mpfile,channel,mpP, [(frame_counter) (frame_counter+nframes_to_cache-1)],the_x);
mpP.Image{trial}=LinescanFramesUnfoldMPScan(data_folded)';
catch
    mpP.Image=0;%zeros(mpP.num_frames*str2num(mpP.Header.Frame_Height),1);
end
try
for chNum = 3%1 : 4
   chNumStr = num2str(chNum);
   if strcmp(mpP.Header.(['Enabled' chNumStr]),'True')
       mpP.(['Ch' chNumStr]) = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpP,[min(trialboundaries) max(trialboundaries)],the_x);
   end
end
mpP.ECoG.Fs=1/length(mpP.Ch3)/str2num(mpP.Header.Frame_Count)*((1/mpP.Header.LineRate)/str2num(mpP.Header.Frame_Height));
end

end
save(['mpP_' fname(1:end-4) '_' the_date],'mpP')

%% dependent functions
function chData = mp2mat_getChannelData_Episodic(mpfile,chNum,mpContent, theframes,the_x,nframes)


%read first data frame and figure out if channel contains analog or image data
tempframe = mpfile.ReadFrameData(chNum,1);
xsize=the_x(2)-the_x(1)+1;
myframes=theframes(1):theframes(2);
if numel(tempframe) == mpContent.xsize * mpContent.ysize
   %got image
    
   chData = zeros(mpContent.ysize,xsize,max(theframes)-min(theframes),'uint16');
    temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
    chData(:,:,1)=temp_frame_reshaped(:,the_x(1):the_x(2));
   for nf = 1:length(myframes)
       tempframe = mpfile.ReadFrameData(chNum,myframes(nf));
        temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
         chData(:,:,nf)=temp_frame_reshaped(:,the_x(1):the_x(2));

   end
else
   %got analog data
   np = length(tempframe);
   chData = zeros( np *  nframes,1,'int16');
   idx = 1 : np;
   chData(idx) = tempframe;
   for nf = theframes(1) :theframes(2)
       tempframe = mpfile.ReadFrameData(chNum,nf)';
       idx = idx + np;
       chData(idx) = tempframe;
   end
end
function [fixed_v]=VelocityCleanUpSTDOutliers(v, thresh)
%finds points outside a threshold velocity and interpolated between last
% and next good points, unless at the begining or end, then puts in mean
% velocities
%pd 2-21-08
upper_bound=thresh*std(v)+mean(v);
lower_bound=-thresh*std(v)+mean(v);
badpoints=union(find(v>upper_bound),find(v<lower_bound));
noutliers=length(badpoints)
fixed_v=v;
while noutliers>0
%badpoints=find(abs(v)>thresh);
goodpoints=union(setdiff(badpoints+1,badpoints),setdiff(badpoints-1,badpoints));
for p=1:length(badpoints)
    lastgood=max(find(goodpoints<badpoints(p)));
    nextgood=min(find(goodpoints>badpoints(p)));
    if ((goodpoints(1)>0)&&(goodpoints(end)<=length(fixed_v)))
        if((numel(lastgood)>0)&&(isempty(nextgood)==0))
        fixed_v(badpoints(p))=.5*fixed_v(goodpoints(lastgood))+.5*fixed_v(goodpoints(nextgood));
        elseif((numel(lastgood)>0))
        fixed_v(badpoints(p))=fixed_v(goodpoints(nextgood)); 
        else
        fixed_v(badpoints(p))=fixed_v(goodpoints(lastgood)); 
        end
    elseif (goodpoints(1)==0)
     fixed_v(badpoints(p))=mean(fixed_v);%(goodpoints(nextgood));
     elseif (goodpoints(end)==length(fixed_v)+1)
     fixed_v(badpoints(p))=mean(fixed_v);%fixed_v(goodpoints(lastgood)); 
    elseif numel(goodpoints)==0
         fixed_v(badpoints(p))=-fixed_v;
    end
    
end
upper_bound=thresh*std(fixed_v)+mean(fixed_v);
lower_bound=-thresh*std(fixed_v)+mean(fixed_v);
badpoints=union(find(fixed_v>upper_bound),find(fixed_v<lower_bound));
noutliers=length(badpoints)

end
function Header = mp2mat_readHeader(mpfile,fname)
%Read all the header information, but do nothing with it
Header.Scan_Mode = invoke(mpfile,'ReadParameter','Scan Mode');
Header.Frame_Width = invoke(mpfile,'ReadParameter','Frame Width');
Header.Frame_Height = invoke(mpfile,'ReadParameter','Frame Height');
Header.Frame_Count = invoke(mpfile,'ReadParameter','Frame Count');
Header.X_Position = invoke(mpfile,'ReadParameter','X Position');
Header.Y_Position = invoke(mpfile,'ReadParameter','Y Position');
Header.Z_Position = invoke(mpfile,'ReadParameter','Z Position');
Header.Stack_Count = invoke(mpfile,'ReadParameter','Stack Count');
Header.Z_Interval = invoke(mpfile,'ReadParameter','z- Interval');
Header.Averaging_Count = invoke(mpfile,'ReadParameter','Averaging Count');
Header.Repeat_Count = invoke(mpfile,'ReadParameter','Repeat Count');
Header.Magnification = invoke(mpfile,'ReadParameter','Magnification');
Header.Rotation = invoke(mpfile,'ReadParameter','Rotation');
Header.X_Frame_Offset = invoke(mpfile,'ReadParameter','X Frame Offset');
Header.Y_Frame_Offset = invoke(mpfile,'ReadParameter','Y Frame Offset');
Header.Channel_Name1 = invoke(mpfile,'ReadParameter','Channel Name (1)');
Header.Channel_Name2 = invoke(mpfile,'ReadParameter','Channel Name (2)');
Header.Channel_Name3 = invoke(mpfile,'ReadParameter','Channel Name (3)');
Header.Channel_Name4 = invoke(mpfile,'ReadParameter','Channel Name (4)');
Header.Enabled1 = invoke(mpfile,'ReadParameter','Enabled (1)');
Header.Enabled2 = invoke(mpfile,'ReadParameter','Enabled (2)');
Header.Enabled3 = invoke(mpfile,'ReadParameter','Enabled (3)');
Header.Enabled4 = invoke(mpfile,'ReadParameter','Enabled (4)');
Header.Input_Range1 = invoke(mpfile,'ReadParameter','Input Range (1)');
Header.Input_Range2 = invoke(mpfile,'ReadParameter','Input Range (2)');
Header.Input_Range3 = invoke(mpfile,'ReadParameter','Input Range (3)');
Header.Input_Range4 = invoke(mpfile,'ReadParameter','Input Range (4)');
Header.Channel_Unit3 = invoke(mpfile,'ReadParameter','Channel Unit (3)');
Header.Channel_Unit4 = invoke(mpfile,'ReadParameter','Channel Unit (4)');
Header.Channel_Prefix3 = invoke(mpfile,'ReadParameter','Channel Prefix (3)');
Header.Channel_Prefix4 = invoke(mpfile,'ReadParameter','Channel Prefix (4)');
Header.Conversion_Factor3 = invoke(mpfile,'ReadParameter','Conversion Factor (3)');
Header.Conversion_Factor4 = invoke(mpfile,'ReadParameter','Conversion Factor (4)');
Header.Offset3 = invoke(mpfile,'ReadParameter','Offset (3)');
Header.Offset4 = invoke(mpfile,'ReadParameter','Offset (4)');
Header.Data_Point_Per_Frame3 = invoke(mpfile,'ReadParameter','Data Point Per Frame (3)');
Header.Data_Point_Per_Frame4 = invoke(mpfile,'ReadParameter','Data Point Per Frame (4)');
Header.Pixel_Clock= invoke(mpfile,'ReadParameter','Pixel Clock');
Header.Frame_Rate=invoke(mpfile,'ReadParameter','Frame Rate');
Header.Filename = fname;
Header.Comments = invoke(mpfile,'ReadParameter','Comments');
function chData = mp2mat_getChannelData_narrow_noplot(mpfile,chNum,mpContent, theframes,the_x)


%read first data frame and figure out if channel contains analog or image data
tempframe = mpfile.ReadFrameData(chNum,1);
xsize=the_x(2)-the_x(1)+1;
myframes=theframes(1):theframes(2);
if numel(tempframe) == mpContent.xsize * mpContent.ysize
   %got image
    
   chData = zeros(mpContent.ysize,xsize,max(theframes)-min(theframes),'uint16');
    temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
    chData(:,:,1)=temp_frame_reshaped(:,the_x(1):the_x(2));
   for nf = 1:length(myframes)
       tempframe = mpfile.ReadFrameData(chNum,myframes(nf));
        temp_frame_reshaped= transpose(reshape(tempframe,mpContent.xsize,mpContent.ysize));
         chData(:,:,nf)=temp_frame_reshaped(:,the_x(1):the_x(2));

   end
else
   %got analog data
   np = length(tempframe);
   chData = zeros( np *  mpContent.num_frames,1,'int16');
   idx = 1 : np;
   chData(idx) = tempframe;
   for nf = 1 :theframes(2)
       tempframe = mpfile.ReadFrameData(chNum,nf)';
       idx = idx + np;
       chData(idx) = tempframe;
   end
end
function [frames_hold]=LinescanFramesUnfold(passedframes,nframes);
%changes mpscan linescan data from frames to a  "scroll"
frame_height=size(passedframes,1);
frame_width=size(passedframes,2);
frames_hold=zeros(frame_height*nframes,frame_width,'uint16');
for k=1:nframes
    frames_hold(((k-1)*frame_height+1):((k)*frame_height),:)=squeeze(passedframes(:,:,k));
end
function [fixed_v]=VelocityCleanUp(v, thresh)
%finds points faster than a threshold velocity and interpolated between last
% and next good points
%pd 11-30-07

badpoints=find(abs(v)>thresh);
goodpoints=union(setdiff(badpoints+1,badpoints),setdiff(badpoints-1,badpoints));
fixed_v=v;
for p=1:length(badpoints)
    lastgood=max(find(goodpoints<badpoints(p)));
    nextgood=min(find(goodpoints>badpoints(p)));
    if ((goodpoints(1)>0)&&(goodpoints(end)<=length(v)))

        if((numel(lastgood)>0)&&(isempty(nextgood)==0))
        fixed_v(badpoints(p))=.5*v(goodpoints(lastgood))+.5*v(goodpoints(nextgood));
        elseif((numel(lastgood)>0))
        fixed_v(badpoints(p))=v(goodpoints(nextgood)); 
        else
        fixed_v(badpoints(p))=v(goodpoints(lastgood)); 
        end
    elseif (goodpoints(1)==0)
     fixed_v(badpoints(p))=v(goodpoints(nextgood));
     elseif (goodpoints(end)==length(v)+1)
     fixed_v(badpoints(p))=v(goodpoints(lastgood)); 
    elseif numel(goodpoints)==0
         fixed_v(badpoints(p))=-v;
    end
    
end
function [frames_hold]=LinescanFramesUnfoldMPScan(passedframes);
%changes mpscan linescan data from frames to a  "scroll"
frame_height=size(passedframes,1);
frame_width=size(passedframes,2);
nframes=size(passedframes,3)
frames_hold=zeros(frame_height*nframes,frame_width,'uint16');
for k=1:nframes
    frames_hold(((k-1)*frame_height+1):((k)*frame_height),:)=squeeze(passedframes(:,:,k));
end

